import struct
import sys
import os.path
import xml.dom.minidom
import tempfile

#Error class to invoke when bad things happen
#class drawerr(Exception):
#    def __init__(self,msg):
#        self.msg=msg

#These are the binary 'type' codes of sol file elements
SOLNUMBER = '\x00'
SOLBOOLEAN = '\x01'
SOLSTRING = '\x02'
SOLOBJECT = '\x03'
SOLNULL = '\x05'
SOLUNDEFINED = '\x06'
SOLARRAY = '\x08'
SOLARRAYEND = '\x09'
SOLDATE = '\x0B'
SOLXML = '\x0F'

#Get command line arguments
argv = sys.argv

#Write the header of the sol file, contains file length
def solputheader(f, name):
	f.write("\x00\xBF\x00\x00\x00\x00TCSO\x00\x04\x00\x00\x00\x00")
	f.write(struct.pack(">H", len(name)) + name + "\x00\x00\x00\x00")
	return

#Write a floating point number element
def solputnumber(f, name, number):
	f.write(struct.pack(">H", len(name)) + name + SOLNUMBER + struct.pack(">d", number))
	return

#Write a string element
def solputstring(f, name, string):
	f.write(struct.pack(">H", len(name)) + name + SOLSTRING + struct.pack(">H", len(string)) + string)
	return

#Start an array
def solputarray(f, name, length):
	f.write(struct.pack(">H", len(name)) + name + SOLARRAY + struct.pack(">L", length))
	return

#Start an 'object'
def solputobject(f, name):
	f.write(struct.pack(">H", len(name)) + name + SOLOBJECT)
	return

#End an array - also ends objects
def solputarrayend(f):
	f.write("\x00\x00\x09")
	return

#Write the correct file size to the header.
#Call this just before saving sol file
def solfixsize(f, filename):
	savepos = f.tell()
	f.seek(0, 2)
	size = f.tell()
	f.seek(2)
	f.write(struct.pack(">L", size - 6))
	f.seek(savepos)
	return

#Writes an array of vectors to a .sol file in the format Linerider uses
def lrwrite(vectors):
	f = tempfile.TemporaryFile()		#Create a temporary file
	solputheader(f, "undefined")		#The file is called 'undefined'
	solputarray(f, "trackList", 1)		#An array 'trackList' of all tracks
	solputobject(f, "0")			#Track 0
	solputarray(f, "data", len(vectors))	#An array of the vectors that make the track

	for j, vector in enumerate(vectors):		#For each vector
		solputarray(f, str(j), 5)		#Create a five element array
		solputnumber(f, "0", vector[0])		#Starting x coord
		solputnumber(f, "1", vector[1])		#Starting y coord
		solputnumber(f, "2", vector[2])		#Ending x coord
		solputnumber(f, "3", vector[3])		#Ending y coord
		solputnumber(f, "4", 0.0)		#Fifth element is 0 (why??)
		solputarrayend(f)			#End the array
		
	solputarrayend(f)			#End the track vector array
	solputstring(f, "label", "track")	#Label it with a string
	solputarrayend(f)			#End the track
	solputarrayend(f)			#End the trackList
	f.write("\x00")				#End of first level object
	solfixsize(f, "undefined.sol")		#Fix the header
	#This changes the console output mode to binary
	#The default is 'text' on windows, for unix it doesn't matter
	if sys.platform == "win32":
                import os, msvcrt
                msvcrt.setmode(sys.stdout.fileno(), os.O_BINARY)
	f.seek(0)
	sys.stdout.write(f.read())		#Write to stdout for saving
	f.close()				#Close the temporary file
	return


#Returns a point on a bezier with start-point p0 and end point p3
#and modifiers p1 and p2
def bezierpoint(p0, p1, p2, p3, t):
	cx = 3.0 * (p1[0] - p0[0])
	bx = 3.0 * (p2[0] - p1[0]) - cx
	ax = p3[0] - p0[0] - cx - bx
	cy = 3.0 * (p1[1] - p0[1])
	by = 3.0 * (p2[1] - p1[1]) - cy
	ay = p3[1] - p0[1] - cy - by

	tSquared = t * t
	tCubed = tSquared * t
	x = (ax * tCubed) + (bx * tSquared) + (cx * t) + p0[0]
	y = (ay * tCubed) + (by * tSquared) + (cy * t) + p0[1]
	
	return (x, y)

#Convert a bezier to an array of straight line segments
def bezier(vectors, p0, p1, p2, p3):
	t = 0.0
	lastpoint = p0
	#The next 3 lines calculate _approximately_ the size of the bezier
	#The longer the bezier, the more segments (detail) we need
	sumx = abs(p0[0] - p1[0]) / 2.0 + abs(p0[0] - p3[0]) + abs(p2[0] - p3[0]) / 2.0
	sumy = abs(p0[1] - p1[1]) / 2.0 + abs(p0[1] - p3[1]) + abs(p2[1] - p3[1]) / 2.0
	detail = 10 / ((sumx * sumx + sumy * sumy)**.5)
	#Check if detail valid
	if(detail > 0.25):
		detail = 0.25
	if(detail < 0.001):
		detail = 0.001
	#Iterate through t = 0 to t = 1 to get complete bezier and append each point
	#to 'vectors' as we go
	while t < 1.0 - detail:
		t += detail
		point = bezierpoint(p0, p1, p2, p3, t)
		vectors.append([lastpoint[0], lastpoint[1], point[0], point[1]])
		#print lastpoint[0], lastpoint[1], point[0], point[1]
		lastpoint = point
	vectors.append([lastpoint[0], lastpoint[1], p3[0], p3[1]])
	#print lastpoint[0], lastpoint[1], p3[0], p3[1]
	return

#Append a straight line to 'vectors'
def line(vectors, p0, p1):
	vectors.append([p0[0], p0[1], p1[0], p1[1]])
	#print p0[0], p0[1], p1[0], p1[1]
	return

#This function takes a path element of an SVG file and interprets
#the data in it. It should contain only lines and beziers.
def parsecommand(vectors, command):
	command = command.split()
	i = 0
	while i < len(command):
		if command[i] == "M":	#Start of a line
			p0 = command[i + 1].split(',')
			p0[0] = float(p0[0])
			p0[1] = float(p0[1])
			i += 1
		if command[i] == "C":	#Bezier
			p1 = command[i + 1].split(',')
			p1[0] = float(p1[0])
			p1[1] = float(p1[1])
			p2 = command[i + 2].split(',')
			p2[0] = float(p2[0])
			p2[1] = float(p2[1])
			p3 = command[i + 3].split(',')
			p3[0] = float(p3[0])
			p3[1] = float(p3[1])
			bezier(vectors, p0, p1, p2, p3)
			p0 = p3
			i += 3
		if command[i] == "L":	#Line
			p1 = command[i + 1].split(',')
			p1[0] = float(p1[0])
			p1[1] = float(p1[1])
			line(vectors, p0, p1)
			p0 = p1
			i += 1
		i += 1

#This is a recursive function to find each path element in a SVG (xml) file
#The path element contains beziers and lines that are appended to our array
#of vectors
def nextxml(vectors, node):
	while node:
		if node.nodeType == 1:
			if node.tagName == "svg" or node.tagName == "g":
				nextxml(vectors, node.firstChild)
			if node.tagName == "path":
				command = node.getAttribute("d").encode("utf-8")
				#print command
				parsecommand(vectors, command)	#interpret command
		node = node.nextSibling
	return

#Open an xml file and start the recursive function 'nextxml' to find
#all occurences of a path element
def parsexml(filename):
	d = xml.dom.minidom.parse(argv[1])
	vectors = []
	nextxml(vectors, d.firstChild)
	d.unlink()
	return vectors
	
#If we have an argument (which should be the filename)
if len(argv) > 1:
	vectors = parsexml(argv[1])	#Get an array of vectors from each '<p>' path element of the file
	lrwrite(vectors)		#Write the vectors in linerider format
else:
	print "\nusage: draw filename.svg\n"

